import click


@click.command()
@click.argument('number', type=int, nargs=1)
def main(number):
	binary_str = "{0:b}".format(number)
	binary_str = binary_str[::-1]
	for i, c in enumerate(binary_str):
		print '{:02d}: {:s}'.format(i, c)


if __name__ == '__main__':
	main()



import re
import binascii


def string_to_hex(s):
	return binascii.hexlify(s)


def hex_to_string(h):
	return binascii.unhexlify(h)


# a.k.a levenshteins distance, see https://en.wikipedia.org/wiki/Edit_distance
def get_edit_distance(str_a, str_b):
	if len(str_a) > len(str_b):
		str_a, str_b = str_b, str_a

	distances = range(len(str_a) + 1)
	for i2, c2 in enumerate(str_b):
		distances_ = [i2 + 1]
		for i1, c1 in enumerate(str_a):
			if c1 == c2:
				distances_.append(distances[i1])
			else:
				distances_.append(1 + min((distances[i1], distances[i1 + 1], distances_[-1])))
		distances = distances_
	return distances[-1]


def get_re_group_n(pattern, _str, n):
	p = re.compile(pattern)
	m = p.match(_str)
	return m.group(n)


def get_re_group_multi(pattern, _str, _list):
	p = re.compile(pattern)
	m = p.match(_str)
	res = [m.group(i) for i in _list]
	return tuple(res)


def contains_re(pattern, _str):
	return bool(re.search(pattern, _str))


def match_re(pattern, _str):
	return bool(re.match(pattern, _str))


if __name__ == '__main__':
	print match_re(r'a+b+$', 'aabbb')



import pysftp
import click


@click.command()
@click.option('-h', '--host', 'host', help='sftp server address, domain or ip')
@click.option('-P', '--port', 'port', type=int, default=22, help='sftp server port')
@click.option('-u', '--user', 'user', help='sftp server login user')
@click.option('-p', '--password', 'password', help='sftp server login password')
@click.argument('src', nargs=1)
@click.argument('dst', nargs=1)
def main(host, port, user, password, src, dst):
    cnopts = pysftp.CnOpts()
    cnopts.hostkeys = None
    with pysftp.Connection(host=host, port=port, username=user, password=password, cnopts=cnopts) as sftp:
        sftp.put(src, dst)


if __name__ == '__main__':
    main()



import click
from clpsz.ufile import read_file_as_list


RESULT = {}
LETTERS = set()


def init_letters():
    for i in range(26):
        LETTERS.add(chr(ord('a') + i))
        LETTERS.add(chr(ord('A') + i))
    LETTERS.add("'")


def process_line(line):
    start = False
    word = ''
    for letter in line:
        if letter in LETTERS:
            word += letter
            if not start:
                start = True
        else:
            if start:
                RESULT[word] = RESULT.get(word, 0) + 1
                start = False
                word = ''


@click.command()
@click.option('-s', '--src', 'src', help='srt file path')
def main(src):
    main1(src)


def main1(src):
    init_letters()
    lines = read_file_as_list(src)
    for line in lines:
        process_line(line)
    for k, v in RESULT.iteritems():
        print("{} {}".format(k, v))


if __name__ == '__main__':
    main1('/tmp/ta.srt')



import os
import sys
import json
import click
from .udatetime import millisecond_timestamp_to_str


@click.command()
def main():
    main1()


def main1():
    k = 0
    try:
        buff = ''
        while True:
            buff += sys.stdin.read(1)
            if buff.endswith(os.linesep):
                line = buff[:-1]
                print(parse_line(line))
                buff = ''
                k = k + 1
    except KeyboardInterrupt:
        sys.stdout.flush()
        pass
    print('processed {} lines'.format(k))


def parse_line(line):
    try:
        data = json.loads(line)
        is_ddl = data['isDdl']
        db = data['database']
        table = data['table']
        _type = data['type']
        ts = data['ts']
        if is_ddl:
            _len = 0
            res = 'ddl {}.{}'.format(db, table)
        else:
            _len = len(data['data'])
            res = '{} | {} rows affected | {} {}.{}'.format(millisecond_timestamp_to_str(int(ts)), _len, _type, db, table)
        return res
    except Exception as e:
        raise Exception(e)


if __name__ == '__main__':
    pass



import os
import re
import sys


def read_file_as_list(file_path):
	"""
	read_file_as_list(file_path) -> list of string

	read a file and convert to a lis tof string, split the file content by new line,
	remove the last line break.
	"""
	res = []
	with open(file_path, 'r') as f:
		for line in f:
			if line[-1] == os.linesep:
				line = line[:-1]
			res.append(line)
	return res


def read_file_as_list_and_keep_linesep(file_path):
	res = []
	with open(file_path, 'r') as f:
		for line in f:
			res.append(line)
	return res


def write_to_file(file_path, content):
	"""
	write_to_file(file_path, content) -> None
	write content to file
	"""
	with open(file_path, 'w') as f:
		f.write(content)


def read_from_file(file_path):
	"""
	write_to_file(file_path, content) -> None
	write content to file
	"""
	with open(file_path, 'r') as f:
		return f.read()


def get_file_line_count(file_path):
	count = 0
	with open(file_path, 'r') as f:
		for _ in f:
			count += 1
	return count


def file_sub(file_path, old, new):
	file_content_list = read_file_as_list_and_keep_linesep(file_path)
	with open(file_path, 'w') as f:
		for line in file_content_list:
			f.write(line.replace(old, new))


def file_sub_re(file_path, pattern, repl):
	pattern = re.compile(pattern)
	file_content_list = read_file_as_list_and_keep_linesep(file_path)
	with open(file_path, 'w') as f:
		for line in file_content_list:
			f.write(pattern.sub(repl, line))


# if delete=False, remove not match lines
# else remove match lines
def filter_file_re(file_path, pattern, delete=False):
	pattern = re.compile(pattern)
	file_content_list = read_file_as_list_and_keep_linesep(file_path)
	with open(file_path, 'w') as f:
		for line in file_content_list:
			if not delete and pattern.search(line):
				f.write(line)
			elif delete and not pattern.search(line):
				f.write(line)


def yield_stdin_by_line():
	k = 0
	try:
		buff = ''
		while True:
			buff += sys.stdin.read(1)
			if buff.endswith(os.linesep):
				line = buff[:-1]
				yield line
				buff = ''
				k = k + 1
	except KeyboardInterrupt:
		sys.stdout.flush()
		pass


if __name__ == '__main__':
	# file_sub_re('/tmp/aaa.txt', r'w.*r ', 'xixi')
	# write_to_file("/tmp/ta", "haha\nxixi\n")

	filter_file_re('/Users/clpsz/Desktop/tmp/teller9.tl9_orgbasic__insert1.sql', "^INSERT INTO")




import os
import errno
import shutil
from multiprocessing import Pool

import pymysql
import click


def read_file_as_list(file_path):
    res = []
    with open(file_path, 'r') as f:
        for line in f:
            if line[-1] == os.linesep:
                line = line[:-1]
            res.append(line)
    return res


def get_column_names(conn, db, table):
    cursor = conn.cursor()
    cursor.execute('USE {}'.format(db))
    cursor.execute('DESC {}'.format(table))
    data = cursor.fetchall()
    cursor.close()
    columns = [e[0] for e in data]
    return columns


def dump_all_wrapper(args):
    dump_all(*args)


def dump_all(conn_param, db_tables, exclude_columns, directory):
    conn = pymysql.connect(
        host=conn_param['host'],
        port=conn_param['port'],
        user=conn_param['user'],
        passwd=conn_param['passwd'],
    )
    for db_table in db_tables:
        db, table = db_table.split('.')
        file_path = '{}/{}-{}_dump.csv'.format(directory, db, table)
        dump(conn, db, table, exclude_columns, file_path)
    conn.close()


def dump(conn, db, table, exclude_columns, path):
    columns = get_column_names(conn, db, table)
    for column in exclude_columns:
        if column in columns:
            columns.remove(column)

    cursor = conn.cursor()
    cursor.execute('USE {}'.format(db))
    cursor.execute('SELECT {} FROM {}'.format(','.join(columns), table))
    data = cursor.fetchall()
    cursor.close()
    with open(path, 'w') as f:
        f.write(','.join(columns) + '\n')
        for row in data:
            f.write(','.join([str(e) for e in row]) + '\n')


def create_dir_if_not_exists(path):
    try:
        if os.path.exists(path):
            shutil.rmtree(path)
        os.makedirs(path)
    except OSError as e:
        if e.errno != errno.EEXIST:
            raise


def split(a, n):
    k, m = divmod(len(a), n)
    return (a[i * k + min(i, m):(i + 1) * k + min(i + 1, m)] for i in range(n))


@click.command()
@click.option('-d', '--db-config-path', 'db_config_path', required=True, help='database config file path, file in json format')
@click.option('-t', '--table-list-path', 'table_list_path', required=True, help='table list file path, file in json format')
@click.option('-o', '--output-dir', 'output_dir', required=True, help='path to output directory')
@click.option('-e', '--exclude-columns', 'exclude_columns', help='columns to exclude, separate by comma')
@click.option('-c', '--concurrent', 'concurrent', type=int, default=1, help='concurrency')
def main(db_config_path, table_list_path, output_dir, exclude_columns, concurrent):
    if exclude_columns is None:
        exclude_columns = {}
    else:
        exclude_columns = set(exclude_columns.split(','))

    create_dir_if_not_exists(output_dir)

    db_config = read_file_as_list(db_config_path)
    host, port, user, password = db_config[0], db_config[1], db_config[2], db_config[3]
    port = int(port)
    conn_param = dict(
        host=host,
        port=port,
        user=user,
        passwd=password,
    )

    table_list = read_file_as_list(table_list_path)
    if concurrent == 1:
        dump_all(conn_param, table_list, exclude_columns, output_dir)
    else:
        parts = split(table_list, concurrent)
        args = []
        for part in parts:
            args.append((conn_param, part, exclude_columns, output_dir))
        p = Pool(concurrent)
        p.map(dump_all_wrapper, args)


if __name__ == '__main__':
    main()



import os
import sys
import json
import click
from confluent_kafka import Consumer, KafkaError


@click.command()
@click.option('-b', '--brokers', 'brokers', required=True, help='kafka brokers')
@click.option('-t', '--topic', 'topic', required=True, help='kafka topic to consume from')
@click.option('-c', '--consumer-group', 'consumer_group', required=True, help='consumer group')
def main(brokers, topic, consumer_group):
    main1(brokers, topic, consumer_group)


def main1(brokers, topic, consumer_group):
    settings = {
        'bootstrap.servers': brokers,
        'group.id': consumer_group,
        'client.id': 'client-1',
        'enable.auto.commit': True,
        'session.timeout.ms': 6000,
        'default.topic.config': {'auto.offset.reset': 'latest'} # latest, none, earliest
    }
    c = Consumer(settings)
    c.subscribe([topic])

    try:
        while True:
            msg = c.poll(0.1)
            if msg is None:
                continue
            elif not msg.error():
                print(msg.value())
            elif msg.error().code() == KafkaError._PARTITION_EOF:
                print('End of partition reached {0}/{1}'.format(msg.topic(), msg.partition()))
            else:
                print('Error occured: {0}'.format(msg.error().str()))

    except KeyboardInterrupt:
        pass

    finally:
        c.close()


if __name__ == '__main__':
    main1('sz.vm2:9094', 'test', 'my-consumer')



import subprocess


def write_to_clipboard(output):
    process = subprocess.Popen('pbcopy', env={'LANG': 'en_US.UTF-8'}, stdin=subprocess.PIPE)
    process.communicate(output.encode('utf-8'))


def read_from_clipboard():
    return subprocess.check_output('pbpaste', env={'LANG': 'en_US.UTF-8'}).decode('utf-8')


if __name__ == '__main__':
    print read_from_clipboard()



from __future__ import print_function

import os


# not include files in subdirectories
def get_files_from_dir(dir_path, full_path=True):
    result = []
    for (_dir, dir_names, filenames) in os.walk(dir_path):
        if not os.path.samefile(_dir, dir_path):
            break
        for filename in filenames:
            if full_path:
                result.append(os.path.join(_dir, filename))
            else:
                result.append(filename)
    return result


# not include files in subdirectories
def get_sub_dirs_from_dir(dir_path, full_path=True):
    result = []
    for (_dir, dir_names, filenames) in os.walk(dir_path):
        if not os.path.samefile(_dir, dir_path):
            break
        for dir_name in dir_names:
            if full_path:
                result.append(os.path.join(_dir, dir_name))
            else:
                result.append(dir_name)
    return result


if __name__ == '__main__':
    for e in get_sub_dirs_from_dir('/Users/clpsz/workspace/database-scripts-ng'):
        print(e)



import shlex
from subprocess import PIPE, Popen


def get_exitcode_stdout_stderr(cmd):
	"""
	Execute the external command and get its exitcode, stdout and stderr.
	"""
	args = shlex.split(cmd)

	sub_p = Popen(args, stdout=PIPE, stderr=PIPE)
	out, err = sub_p.communicate()
	exitcode = sub_p.returncode

	return exitcode, out, err



from datetime import datetime


def str_to_timestamp(time_str):
	return int(datetime.strptime(time_str, '%Y-%m-%d %H:%M:%S').strftime("%s"))


def timestamp_to_str(timestamp):
	return datetime.fromtimestamp(timestamp).strftime("%Y-%m-%d %H:%M:%S")


def millisecond_timestamp_to_str(m_timestamp):
	return datetime.fromtimestamp(m_timestamp/1000).strftime("%Y-%m-%d %H:%M:%S") + '.{:03}'.format(m_timestamp % 1000)


def iso_to_str(timestamp):
	d = datetime.strptime(timestamp, "%Y-%m-%dT%H:%M:%S.%fZ")
	return d.strftime("%Y-%m-%d %H:%M:%S")


def get_today_begin_timestamp():
	today = datetime.today().replace(hour=0, minute=0, second=0, microsecond=0)
	return int(today.strftime("%s"))


def get_id_now_timestamp():
	return int(datetime.now().strftime("%s")) + 7*3600


if __name__ == "__main__":
	print(millisecond_timestamp_to_str(1610455370620))



import os
import sys
from setuptools import setup, find_packages


here = os.path.abspath(os.path.dirname(__file__))


# 'setup.py publish' shortcut.
if sys.argv[-1] in ['publish', 'package']:
	os.system('rm -f dist/*')
	os.system('python setup.py sdist')
	if sys.argv[-1] == 'publish':
		os.system('twine upload --repository-url https://upload.pypi.org/legacy/ dist/*')
	sys.exit()


about = {}
with open(os.path.join(here, 'clpsz', '__version__.py'), 'r') as f:
	exec(f.read(), about)

with open('README.md', 'r') as f:
	readme = f.read()


requires = [
	'click',
	'PyMySQL',
	'pysftp',
	'confluent_kafka',
]


setup(
	name=about['__title__'],
	version=about['__version__'],
	description=about['__description__'],
	long_description=readme,
	long_description_content_type='text/markdown',
	author=about['__author__'],
	author_email=about['__author_email__'],
	url=about['__url__'],
	license=about['__license__'],

	packages=[
		'clpsz',
	],
	package_dir={
		'clpsz': 'clpsz'
	},
	package_data={},
	include_package_data=True,

	install_requires=requires,
	entry_points={
		'console_scripts': [
			'clpsz-flag=clpsz.clpsz_flag:main',
			'clpsz-db-export=clpsz.clpsz_db_export:main',
			'clpsz-sftp=clpsz.clpsz_sftp:main',
			'clpsz-kafka-consumer=clpsz.clpsz_kafka_consumer:main',
			'clpsz-canal-parser=clpsz.clpsz_canal_parser:main',
			'clpsz-word-count=clpsz.clpsz_word_count:main',
		]
	}
)
